import datetime
from application import db
from application.forms  import CommentForm
from application.models.users import *
from application.models.applications import Application, Skill, ReviewersApplications, Comment

from flask import render_template, redirect, url_for, request, Blueprint
from flask.ext.security import login_required, roles_accepted
from flask.ext.security.core import current_user


applications = Blueprint('applications', __name__)

@applications.route('/')
@roles_accepted('bfct_board', 'bfct_manager', 'admin')
def index():
    return render_template('applications/index.html', 
        title='applications',
        applications=Application.query.all())

@applications.route('/view/<int:id>')
@login_required
@roles_accepted('bfct_board', 'bfct_manager', 'admin')
def view(id):

    comment_form = CommentForm()

    review = ReviewersApplications.query.\
        filter_by(application_id=id).\
        filter_by(reviewer_blender_id=current_user.id).\
        first()

    reviews = ReviewersApplications.query.\
        filter_by(application_id=id).\
        all()

    return render_template('applications/view.html', 
        title='applications',
        application=Application.query.get_or_404(id),
        review=review,
        reviews=reviews,
        comment_form=comment_form)

@applications.route('/vote/<int:approved>/<int:id>')
@roles_accepted('bfct_board', 'bfct_manager', 'admin')
def vote(approved, id):
    application = Application.query.get_or_404(id)
    review = ReviewersApplications.query.\
        filter_by(application_id=id).\
        filter_by(reviewer_blender_id=current_user.id).\
        first()

    if approved:
        if review:
            application.reject -= 1
        application.approve += 1
    else:
        if review:
            application.approve -= 1
        application.reject += 1

    if application.status == 'submitted':
        application.status = 'under_review'
        application.review_start_date = datetime.datetime.now()
    db.session.add(application)

    if not review:
        review = ReviewersApplications(
            application_id=id,
            reviewer_blender_id=current_user.id,
            approved=approved) 
    else:
        review.approved = approved

    db.session.add(review)
    
    db.session.commit()
    return redirect(url_for('.view', id=id))

@applications.route('/final-review/<int:approved>/<int:id>')
@roles_accepted('bfct_manager', 'admin')
def final_review(approved, id):
    application = Application.query.get_or_404(id)

    if application.status != 'under_review':
        return 'error'
    else:
        if approved:
            application.status = 'approved'
        else:
            application.status = 'rejected'
        application.review_end_date = datetime.datetime.now()
    db.session.add(application)
   
    db.session.commit()
    return redirect(url_for('.view', id=id))


@applications.route('/comment/<int:application_id>', methods=['POST'])
@roles_accepted('bfct_board', 'bfct_manager', 'admin')
def comment(application_id):
    comment_form = CommentForm()
    if comment_form.validate_on_submit():
        comment = Comment(
            blender_id=current_user.id,
            application_id=application_id,
            text=comment_form.text.data)

        db.session.add(comment)
        db.session.commit()
    return redirect(url_for('.view', id=application_id))

